/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 *  
 * Copyright (C) 2004 - 2012 Cassidian, an EADS company
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */
package org.ow2.weblab.core.helper.impl;

import java.io.StringWriter;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ow2.weblab.core.extended.exception.WebLabCheckedException;
import org.ow2.weblab.core.extended.util.PoKUtil;
import org.ow2.weblab.core.model.PieceOfKnowledge;

import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;

/**
 * Internal class to read/write couples of (predicate, values)
 * 
 * @author asaval
 *
 */
public class AdvancedLinkedHashMap implements IPredicateValuePairs {


	/**
	 * Map of couples of (predicate, values)
	 */
	private LinkedHashMap<String, List<WTriple>> map = new LinkedHashMap<String, List<WTriple>>();

	/**
	 * the Statement containing triples 
	 */
	private final Statements wtm;

	/**
	 * temporary uri used to create new WTriple
	 */
	private String temporaryURI = null;


	/**
	 * Internal constructor to aggregate Statements
	 * @param wtm the Statements
	 * @param uri the uri for new WTriple
	 */
	protected AdvancedLinkedHashMap(final Statements wtm, final String uri) {
		this.wtm = wtm;
		this.temporaryURI = uri;
	}

	/**
	 * Constructor from Statements
	 * @param wtm the Statements
	 */
	public AdvancedLinkedHashMap(final Statements wtm) {
		this.wtm = wtm;
	}

	/**
	 * Map of couples of (predicate, values)
	 * @return the Map of couples of (predicate, values)
	 */
	public LinkedHashMap<String, List<WTriple>> getMap() {
		return this.map;
	}


	/**
	 * Set the map of couples of (predicate, values)
	 * @param map of couples of (predicate, values)
	 */
	public void setMap(final LinkedHashMap<String, List<WTriple>> map) {
		this.map = map;
	}


	@Override
	public Object getValue(final String predicate) {
		final List<WTriple> wts = this.map.get(predicate);
		if ((wts == null) || wts.isEmpty()) {
			return null;
		}
		if (wts.size() == 1) {
			return wts.get(0).getObject();
		}
		final List<Object> list = new LinkedList<Object>();
		for (final WTriple wtriple : wts) {
			list.add(wtriple.getObject());
		}
		return list;
	}


	@Override
	public void writeStatements(final String[] predicates, final String[] objects, PieceOfKnowledge pokover) throws WebLabCheckedException {
		if ((predicates == null) || (objects == null) || (predicates.length == 0) || (objects.length == 0) || (predicates.length != objects.length)) {
			throw new WebLabCheckedException("Unable to write predicate and object.");
		}

		Collection<WTriple> wls = this.getStatements();
		if (wls.isEmpty()) {
			if (this.temporaryURI != null) {
				wls = new LinkedList<WTriple>();
				// Create a dummy WTriple
				wls.add(new WTriple(
						new Triple(Node.createURI(this.temporaryURI),Node.createURI("mnone"),Node.createLiteral("none")),
						"hop", pokover, new LinkedList<org.ow2.weblab.core.model.Resource>()));
				
				this.temporaryURI = null;
			} else {
				throw new WebLabCheckedException("Unable to write predicate and object because there is no previous element.");
			}
		}

		final WTriple wtriple = wls.iterator().next();

		PieceOfKnowledge pok = pokover;
		if (pok == null) {
			pok = wtriple.getAnnotation();
		}

		final List<WTriple> list = new LinkedList<WTriple>();

		final String uri = wtriple.getSubject();
		for (int index = 0; index < predicates.length; index++) {
			final WTriple nwtriple = new WTriple(uri, predicates[index], objects[index], wtriple.getSubjectResource(), this.wtm.getResource(objects[index]), wtriple.getAnnotatedOn());
			nwtriple.setTriple(wtriple.getTriple());
			nwtriple.setAnnotation(pok);
			this.put(nwtriple);
			list.add(nwtriple);
		}

		AdvancedLinkedHashMap.write(list, pok, this.wtm, new HashMap<String, String>());

	}


	protected static void write(final List<WTriple> list, final PieceOfKnowledge pokover, final Statements wtm, final Map<String, String> mapping)
			throws WebLabCheckedException {

		final Object data = pokover.getData();

		Model model;
		if ((data == null) || !(data instanceof org.w3c.dom.Node)) {
			model = ModelFactory.createDefaultModel();
		} else {
			model = TripleSelectors.loadModel((org.w3c.dom.Node) data);
		}

		for (final WTriple wtriple : list) {
			boolean res = false;
			final String uri = wtriple.getSubject();
			final String predicate = wtriple.getPredicate();
			final String object = wtriple.getObject();
			if ((uri == null) || (predicate == null) || (object == null)) {
				throw new WebLabCheckedException("Unable to write predicate and object : " + predicate + " : " + object + " on " + uri);
			}
			for (final String namespace : wtm.namespaces) {
				if (object.startsWith(namespace)) {
					res = true;
					break;
				}
			}

			final Triple triple = wtriple.getTriple();
			if (triple != null) {
				final Node node = triple.getObject();
				if ((node != null) && !node.isLiteral()) {
					res = true;
				}
			}

			String muri = mapping.get(uri);
			String mpredicate = mapping.get(predicate);
			String mobject = mapping.get(object);
			muri = muri == null ? uri : muri;
			mpredicate = mpredicate == null ? predicate : mpredicate;
			mobject = mobject == null ? object : mobject;

			final Resource subj = model.createResource(muri);
			final Property pred = model.createProperty(mpredicate);

			if (res) {
				model.createStatement(subj, pred, mobject);
			} else {

				final Statement statement = model.createStatement(subj, pred, triple.getObject().getLiteral().getValue().toString());
				model.add(statement);
			}
		}

		final StringWriter swriter = new StringWriter();
		model.write(swriter);
		PoKUtil.setPoKData(pokover, swriter.toString());
		model.close();
	}


	@Override
	public void writeStatement(final String predicate, final String object) throws WebLabCheckedException {
		this.writeStatements(new String[] { predicate }, new String[] { object }, null);
	}


	/**
	 * Inner function to add a WTriple
	 * 
	 * @param wtriple
	 *            a WTriple
	 */
	protected void put(final WTriple wtriple) {
		List<WTriple> list = this.map.get(wtriple.getPredicate());
		if (list == null) {
			list = new LinkedList<WTriple>();
		}
		list.add(wtriple);
		this.map.put(wtriple.getPredicate(), list);
	}


	@Override
	public Set<String> allPredicates() {
		return this.map.keySet();
	}


	@Override
	public Collection<WTriple> getStatements() {
		final List<WTriple> list = new LinkedList<WTriple>();
		for (final List<WTriple> listWTriple : this.map.values()) {
			list.addAll(listWTriple);
		}
		return list;

	}


	@Override
	public String toString() {
		return this.map.toString();
	}
}
