/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 *  
 * Copyright (C) 2004 - 2012 Cassidian, an EADS company
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */

package org.ow2.weblab.core.helper.impl;

import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.ow2.weblab.core.extended.ontologies.RDF;
import org.ow2.weblab.core.model.Annotation;
import org.ow2.weblab.core.model.ComposedResource;
import org.ow2.weblab.core.model.Document;
import org.ow2.weblab.core.model.MediaUnit;
import org.ow2.weblab.core.model.PieceOfKnowledge;
import org.ow2.weblab.core.model.Resource;
import org.ow2.weblab.core.model.ResultSet;
import org.xml.sax.SAXParseException;

import com.hp.hpl.jena.graph.Graph;
import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.rdf.arp.DOM2Model;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.util.iterator.ExtendedIterator;

/**
 * Selectors Helper to ease annotations retrieval.
 * Annotations triples are referenced by WTriple in which
 * subject and object can be references to WebLab Resources.
 * By default, it transforms reified triples as normal triples.
 * It is useful if you just bother on data selection.
 * Example:<br/>
 * <code>
 * 		List&lt;WTriple&gt; results = findInResource(myResource, null, "http://www.weblab-project.org/gld-country", null); 
 * </code>
 * 
 * @author WebLab Team
 * @date 2009-03-11
 */
public class TripleSelectors {


	/**
	 * Limit to first level
	 */
	public boolean limit = false;


	/**
	 * Enable debug mode
	 */
	public static boolean debug = false;


	/**
	 * Support null data
	 */
	public boolean supportNullData = false;


	/**
	 * multi thread support
	 */
	public static boolean multi = false;


	final Object lock = new Object();


	final List<WTriple> syncList = Collections.synchronizedList(new LinkedList<WTriple>());


	private final static int NUMBER_OF_PROC = Runtime.getRuntime().availableProcessors();


	private final static int MULTIPLIER = 1;


	int numberOfJob = 0;

	/**
	 * List of current uris 
	 */
	public transient String[] uris = new String[0];


	/**
	 * Limit annotation search to first level 
	 * @param theLimit true to limit
	 */
	public void limitToFirstLevelAnnotation(final boolean theLimit) {
		this.limit = theLimit;
	}

	/**
	 * Create a TripleSelectors
	 */
	public TripleSelectors() {
		super();
	}

	/**
	 * Create a TripleSelectors with support for null data and list of uris to interpret as resources
	 * @param supportNullData true to support null data
	 * @param uris list of uris to interpret as resources
	 */
	public TripleSelectors(final boolean supportNullData, final String... uris) {
		this.supportNullData = supportNullData;
		if (uris == null) {
			return;
		}
		this.uris = uris;
	}


	/**
	 * Select Triplet (Subject, Predicate, Object) in the resource and
	 * its sub elements (Segments, Annotations, sub-MediaUnit ...).
	 * If subject, predicate or object is null, it acts as a wildcard parameter.
	 * Here is an example how to retrieve all Triplets which match the property "http://www.weblab-project.org/gld-country" <br/>
	 * <br/>
	 * <code>
	 * 		List&lt;WTriple&gt; results: findInResource(myResource, null, "http://www.weblab-project.org/gld-country", null);
	 * </code>
	 * 
	 * @param subject
	 *            the subject that should match in the triplet (can be null)
	 * @param predicate
	 *            the predicate that should match in the triplet (can be null)
	 * @param object
	 *            the object that should match in the triplet (can be null)
	 * @return a list of WTriple that match the subject, predicate and object.
	 */
	public List<WTriple> findInResource(final Resource resource, final String subject, final String predicate, final String object) {
		final Node sub = subject == null ? Node.ANY : Node.createURI(subject);
		final Node pred = predicate == null ? Node.ANY : Node.createURI(predicate);

		boolean buri = false;
		for (final String uri : this.uris) {
			if ((object != null) && object.startsWith(uri)) {
				buri = true;
				break;
			}
		}

		final Node obj = object == null ? Node.ANY : buri ? Node.createURI(object) : Node.createLiteral(object);
		final List<WTriple> list = this.findInResource(resource, sub, pred, obj, new LinkedList<Resource>());
		if (TripleSelectors.multi) {
			while (this.numberOfJob > 0) {
				Thread.yield();
				try {
					Thread.sleep(2);
				} catch (final InterruptedException exception) {
					// Nothing to do
				}
			}
			list.addAll(this.syncList);
			this.syncList.clear();
		}
		return this.aggregateReified(list);
	}


	/**
	 * Select WTriple in resource according to given subject, predicate and object.
	 * Null parameters act as wildcard. It will recursively loop through Resource structure
	 * (children and their annotations) to retrieve RDF in annotations.
	 * 
	 * @param resource
	 *            a WebLab Resource on which there are annotations containing RDF
	 * @param subject
	 *            the subject that should match in the triplet (can be null)
	 * @param predicate
	 *            the predicate that should match in the triplet (can be null)
	 * @param object
	 *            the object that should match in the triplet (can be null)
	 * @param reif
	 *            if true, reified triple will not be aggregated
	 * @return a list of WTriple that match the subject, predicate and object.
	 */
	public List<WTriple> findInResource(final Resource resource, final String subject, final String predicate, final String object, final boolean reif) {
		final Node sub = subject == null ? Node.ANY : Node.createURI(subject);
		final Node pred = predicate == null ? Node.ANY : Node.createURI(predicate);

		boolean buri = false;
		for (final String uri : this.uris) {
			if ((object != null) && object.startsWith(uri)) {
				buri = true;
				break;
			}
		}

		final Node obj = object == null ? Node.ANY : buri ? Node.createURI(object) : Node.createLiteral(object);
		if (reif) {
			final List<WTriple> list = this.findInResource(resource, sub, pred, obj, new LinkedList<Resource>());
			if (TripleSelectors.multi) {
				list.addAll(this.syncList);
				this.syncList.clear();
			}
			return this.aggregateReified(list);
		}
		final List<WTriple> list = this.findInResource(resource, sub, pred, obj, new LinkedList<Resource>());
		if (TripleSelectors.multi) {
			list.addAll(this.syncList);
			this.syncList.clear();
		}
		return list;
	}


	/**
	 * Select WTriple in the resource according to constrains in a filter.
	 * 
	 * @param resource
	 *            a WebLab Resource on which there are annotations containing RDF
	 * @param filter
	 *            a set of constrains
	 */
	public List<WTriple> findInResource(final Resource resource, final Filter filter) {
		final List<WTriple> list = this.findInResource(resource, null, null, null);
		if (filter == null) {
			return list;
		}
		final Set<WTriple> res = new HashSet<WTriple>();
		for (final WTriple wtriple : list) {
			if (!res.contains(wtriple) && filter.matches(wtriple)) {
				res.add(wtriple);
			}
		}
		list.clear();
		list.addAll(res);
		return list;
	}


	/**
	 * Aggregate reified triple in normal ones
	 * 
	 * @param list
	 *            a list of WTriple reified or not
	 * @return a list of WTriple not reified.
	 */
	private List<WTriple> aggregateReified(final List<WTriple> list) {
		final List<WTriple> res = new LinkedList<WTriple>(list);
		final Map<String, List<WTriple>> buffer = new LinkedHashMap<String, List<WTriple>>();
		for (final WTriple wtriple : list) {
			final String pred = wtriple.getPredicate();
			if (pred.startsWith("http://www.w3.org/1999/02/22-rdf-syntax-ns#")) {
				List<WTriple> inner = buffer.get(wtriple.getSubject());
				if (inner == null) {
					inner = new LinkedList<WTriple>();
				}
				inner.add(wtriple);
				buffer.put(wtriple.getSubject(), inner);
				// res.remove(wt);
			}
		}

		for (final Entry<String, List<WTriple>> entry : buffer.entrySet()) {
			String subject = null;
			String predicate = null;
			String object = null;
			Resource subjectr = null;
			Resource objectr = null;
			final List<WTriple> later = new LinkedList<WTriple>();
			for (final WTriple wt : entry.getValue()) {
				final String pred = wt.getPredicate();
				if ("http://www.w3.org/1999/02/22-rdf-syntax-ns#predicate".equals(pred)) {
					predicate = wt.getObject();
				} else if ("http://www.w3.org/1999/02/22-rdf-syntax-ns#object".equals(pred)) {
					object = wt.getObject();
					objectr = wt.getObjectResource();
				} else if ("http://www.w3.org/1999/02/22-rdf-syntax-ns#subject".equals(pred)) {
					subject = wt.getObject();
					subjectr = wt.getObjectResource();
				} else if (!"http://www.w3.org/1999/02/22-rdf-syntax-ns#type".equals(pred)) {
					later.add(wt);
				}
			}
			if ((subject != null) && (predicate != null) && (object != null)) {
				final WTriple wtriple = new WTriple(subject, predicate, object, subjectr, objectr, null);
				res.add(wtriple);
			}
			for (final WTriple wt : later) {
				res.add(new WTriple(subject, wt.getPredicate(), wt.getObject(), subjectr, wt.getObjectResource(), wt.getAnnotatedOn()));
			}
		}
		return res;
	}


	private List<WTriple> findInResource(Resource resource, final Node subject, final Node predicate, final Node object, final List<Resource> resources) {
		final List<WTriple> list = new LinkedList<WTriple>();

		Resource mockResource = resource;
		if (this.limit) { // limit to the first level only
			
			if (mockResource instanceof PieceOfKnowledge) {
				final PieceOfKnowledge pok = new PieceOfKnowledge();
				pok.setData(((PieceOfKnowledge) mockResource).getData());
				mockResource = pok;
			} else {
				mockResource = new Resource();
			}
			mockResource.setUri(resource.getUri());
			final List<Annotation> annotations = new LinkedList<Annotation>();

			// limit to first level annotation too
			for (final Annotation annotation : resource.getAnnotation()) {
				final Annotation innerAnntation = new Annotation();
				innerAnntation.setUri(annotation.getUri());
				innerAnntation.setData(annotation.getData());
				annotations.add(innerAnntation);
			}

			mockResource.getAnnotation().addAll(annotations);
		}

		if (mockResource instanceof ComposedResource) {
			list.addAll(this.find((ComposedResource) mockResource, subject, predicate, object, resources));

		} else if (mockResource instanceof MediaUnit) {
			list.addAll(this.find((MediaUnit) mockResource, subject, predicate, object, resources));
		} else if (mockResource instanceof ResultSet) {
			list.addAll(this.find((ResultSet) mockResource, subject, predicate, object, resources));
		} else if (mockResource instanceof PieceOfKnowledge) {
			list.addAll(this.find((PieceOfKnowledge) mockResource, subject, predicate, object, null, mockResource, resources));
		} else {
			for (final Annotation annotation : mockResource.getAnnotation()) {
				list.addAll(this.find(annotation, subject, predicate, object, null, mockResource, resources));
			}
		}

		return list;
	}


	/**
	 * Copies a list of resources and add an object
	 * 
	 * @param resources
	 *            list of resources
	 * @param resource
	 *            a resource
	 * @return a copy the list and add the resource r
	 */
	protected List<Resource> copyAndAdd(final List<Resource> resources, final Resource resource) {
		final List<Resource> copie = new LinkedList<Resource>();
		copie.addAll(resources);
		copie.add(resource);
		return copie;
	}


	private List<WTriple> find(final ResultSet resultset, final Node subject, final Node predicate, final Node object, final List<Resource> resources) {
		final List<WTriple> list = new LinkedList<WTriple>();

		final PieceOfKnowledge pok = resultset.getPok();
		if (pok != null) {
			list.addAll(this.find(pok, subject, predicate, object, null, resultset, resources));
		}

		for (final Resource resource : resultset.getResource()) {
			list.addAll(this.findInResource(resource, subject, predicate, object, this.copyAndAdd(resources, resource)));
		}

		for (final Annotation annotation : resultset.getAnnotation()) {
			list.addAll(this.find(annotation, subject, predicate, object, null, resultset, this.copyAndAdd(resources, annotation)));
		}
		return list;
	}


	private List<WTriple> find(final ComposedResource composedResource, final Node subject, final Node predicate, final Node object, final List<Resource> resources) {
		final List<WTriple> list = new LinkedList<WTriple>();

		for (final Resource resource : composedResource.getResource()) {
			list.addAll(this.findInResource(resource, subject, predicate, object, this.copyAndAdd(resources, resource)));
		}

		for (final Annotation annotation : composedResource.getAnnotation()) {
			list.addAll(this.find(annotation, subject, predicate, object, null, composedResource, this.copyAndAdd(resources, annotation)));
		}
		return list;
	}



	private List<WTriple> find(final MediaUnit mediaunit, final Node subject, final Node predicate, final Node object, final List<Resource> resources) {
		final List<WTriple> list = new LinkedList<WTriple>();

		resources.add(mediaunit);

		if (mediaunit instanceof Document) {
			final Document doc = (Document) mediaunit;
			for (final MediaUnit mu_in_doc : doc.getMediaUnit()) {
				list.addAll(this.find(mu_in_doc, subject, predicate, object, resources));
			}
		}


		for (final Annotation a : mediaunit.getAnnotation()) {
			list.addAll(this.find(a, subject, predicate, object, null, mediaunit, resources));
		}

		return list;
	}



	/**
	 * Select RDF Triplet in an annotation
	 * 
	 * @param annotation
	 *            an annotation
	 * @param subject
	 *            the subject that should match in the triplet (can be null)
	 * @param predicate
	 *            the predicate that should match in the triplet (can be null)
	 * @param object
	 *            the object that should match in the triplet (can be null)
	 * @param text
	 *            the text associated with the annotation if any
	 * @param resources
	 *            the parent resources
	 * @return a list of triplet which matches the data in the annotation.
	 */
	protected List<WTriple> find(final PieceOfKnowledge annotation, final Node subject, final Node predicate, final Node object, final String text,
			final Resource annotated, final List<Resource> resources) {

		if (TripleSelectors.multi && (this.numberOfJob < (TripleSelectors.MULTIPLIER * TripleSelectors.NUMBER_OF_PROC))) {
			return this.findMT(annotation, subject, predicate, object, text, annotated, resources);
		}

		final Object node = annotation.getData();
		if (this.supportNullData && (node == null)) {
			return new LinkedList<WTriple>();
		}

		resources.add(annotation);
		final List<WTriple> list = this.applySelection((org.w3c.dom.Node) node, subject, predicate, object, text, annotated, resources, annotation);

		for (final Annotation annot : annotation.getAnnotation()) {
			list.addAll(this.find(annot, subject, predicate, object, null, annotated, this.copyAndAdd(resources, annot)));
		}
		return list;
	}


	protected List<WTriple> findMT(final PieceOfKnowledge annotation, final Node subject, final Node predicate, final Node object, final String text,
			final Resource annotated, final List<Resource> resources) {
		synchronized (this.lock) {
			this.numberOfJob++;
		}

		final Thread worker = new Thread("MTFind#" + this.numberOfJob) {


			@Override
			public void run() {
				final List<WTriple> list = new LinkedList<WTriple>();

				final Object node = annotation.getData();
				if (TripleSelectors.this.supportNullData && (node == null)) {
					synchronized (TripleSelectors.this.lock) {
						TripleSelectors.this.numberOfJob--;
					}
					return;
				}

				resources.add(annotation);
				list.addAll(TripleSelectors.this.applySelection((org.w3c.dom.Node) node, subject, predicate, object, text, annotated, resources, annotation));

				for (final Annotation annot : annotation.getAnnotation()) {
					list.addAll(TripleSelectors.this.find(annot, subject, predicate, object, null, annotated, TripleSelectors.this.copyAndAdd(resources, annot)));
				}
				synchronized (TripleSelectors.this.lock) {
					TripleSelectors.this.syncList.addAll(list);
					TripleSelectors.this.numberOfJob--;
				}
			}
		};
		worker.start();

		return new LinkedList<WTriple>();
	}


	/**
	 * Load a model really fast
	 * 
	 * @param data
	 *            a DOM
	 * @return a Jena Model
	 */
	protected static Model loadModel(final org.w3c.dom.Node data) {
		final Model model = ModelFactory.createDefaultModel();
		try {
			final DOM2Model domModel = DOM2Model.createD2M(RDF.NAMESPACE, model);
			domModel.load(data.getFirstChild());
			domModel.close();
		} catch (final SAXParseException exception) {
			exception.printStackTrace();
		}
		return model;
	}


	protected List<WTriple> applySelection(final org.w3c.dom.Node data, final Node subject, final Node predicate, final Node object, final String text, final Resource annotated,
			final List<Resource> resources, final PieceOfKnowledge annotation) {
		final List<WTriple> triples = new LinkedList<WTriple>();

		final Model model = TripleSelectors.loadModel(data);
		final Graph graph = model.getGraph();

		final ExtendedIterator<?> iterator = graph.find(subject, predicate, object);
		while (iterator.hasNext()) {
			final Object obj = iterator.next();
			if (obj instanceof Triple) {
				final Triple triple = (Triple) obj;
				final Node node = triple.getSubject();
				final WTriple wtriple = new WTriple((Triple) obj, text, annotated, this.copyAndAdd(resources, annotation));
				wtriple.setAnnotation(annotation);
				if (node.isBlank()) {
					final ExtendedIterator<?> innerIterator = graph.find(null, null, node);
					while (innerIterator.hasNext()) {
						final Object innerObject = innerIterator.next();
						if (innerObject instanceof Triple) {
							wtriple.setReifiedWTriple(new WTriple((Triple) innerObject, text, annotated, resources));
						}
					}
				}
				triples.add(wtriple);
			}
		}
		model.close();

		return triples;
	}
}
