package org.ow2.weblab.core.helper.impl;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;

import junit.framework.Assert;

import org.junit.BeforeClass;
import org.junit.Test;
import org.ow2.weblab.core.extended.exception.WebLabCheckedException;
import org.ow2.weblab.core.extended.jaxb.WebLabMarshaller;
import org.ow2.weblab.core.extended.ontologies.RDFS;
import org.ow2.weblab.core.helper.test.ResourcesCreator;
import org.ow2.weblab.core.model.Annotation;
import org.ow2.weblab.core.model.Document;
import org.ow2.weblab.core.model.PieceOfKnowledge;
import org.ow2.weblab.core.model.Resource;

import com.hp.hpl.jena.graph.Node;
import com.hp.hpl.jena.graph.Triple;

public class AdvancedLinkedHashMapTest {

	@BeforeClass
	public static void buildResources() throws WebLabCheckedException {
		if (!ResourcesCreator.DOCUMENT.exists()) {
			ResourcesCreator.buildTestResources();
		}
	}


	@Test
	public void testConstruction() throws WebLabCheckedException {
		if (!ResourcesCreator.DOCUMENT.exists()) {
			ResourcesCreator.buildTestResources();
		}
		
		final Document doc = new WebLabMarshaller().unmarshal(ResourcesCreator.DOCUMENT, Document.class);
		final Annotation annotation = new WebLabMarshaller().unmarshal(ResourcesCreator.ANNOTATION, Annotation.class);
		
		final AdvancedSelector rs = RDFSelectorFactory.getSelector(true);
		Statements sts_doc = rs.searchFor(doc);
		Statements sts_ann = rs.searchFor(annotation);
		
		Assert.assertEquals(2, sts_doc.size());
		Assert.assertEquals(1, sts_ann.size());
		
		Assert.assertNull(sts_ann.getFirstValue("weblab://i/DOCUMENT", RDFS.LABEL, Locale.FRENCH));
		Assert.assertEquals("empty",sts_ann.getFirstValue("http://www.w3.org/1999/02/empty", RDFS.LABEL, Locale.FRENCH));
		
		sts_ann.writeAndAdd(sts_doc, annotation);
		
		Assert.assertEquals(2, sts_ann.size());
		
		try{
			sts_doc.writeAndAdd(null, annotation);
			Assert.fail("Can not ad null statements");
		}catch (WebLabCheckedException e) {
			// Normal
		}
		
		try{
			sts_doc.writeAndAdd(sts_ann, null);
			Assert.fail("Can not add null pok");
		}catch (WebLabCheckedException e) {
			// Normal
		}
		
		sts_ann.writeAndAdd(sts_doc, annotation,
				"weblab://i/DOCUMENT","weblab://i/DOCUMENT-Z", // replace subject
				"http://www.w3.org/2000/01/rdf-schema#label","http://myuber#label",// replace property
				"Document2","Document3"); // replace object 
		
		sts_ann = rs.searchFor(annotation);
		String label = sts_ann.getTypedValue("weblab://i/DOCUMENT-Z", "http://myuber#label", String.class);
		Assert.assertNotNull(label);
		
		// won't do anything
		sts_ann.writeAndAdd(sts_doc, annotation,"http://www.w3.org/2000/01/rdf-schema#label");
		
		Assert.assertNotNull(sts_ann.getPokFor("weblab://i/DOCUMENT-Z"));
		
		Assert.assertNotNull(sts_ann.getTypedValue("http://www.w3.org/1999/02/empty", RDFS.LABEL, String.class));
		
		WTriple wt = new WTriple("weblab://test", "weblab://test", "weblab://test", doc, null, annotation);
		wt.setReifiedWTriple(wt);
		sts_ann.add(wt);
		
		Assert.assertEquals(4, sts_ann.size());
		
		sts_ann.clear();
		
		Assert.assertEquals(0, sts_ann.size());
		
		Assert.assertNull(sts_ann.getFirstValue("http://www.w3.org/1999/02/empty", RDFS.LABEL, Locale.FRENCH));
		
		final SimpleSelector ss = RDFSelectorFactory.getSelector();
		Results results = ss.select(doc);
		
		Assert.assertEquals("Document2", results.getTypedValue("weblab://i/DOCUMENT", RDFS.LABEL, String.class));
		
	}
	
	@Test
	public void testWrite() throws WebLabCheckedException {
		if (!ResourcesCreator.DOCUMENT.exists()) {
			ResourcesCreator.buildTestResources();
		}
		final Document doc = new WebLabMarshaller().unmarshal(ResourcesCreator.DOCUMENT, Document.class);
		final Annotation annotation = new WebLabMarshaller().unmarshal(ResourcesCreator.ANNOTATION, Annotation.class);
		
		final AdvancedSelector rs = RDFSelectorFactory.getSelector(true, "uri://uri");
		Statements sts_doc = rs.searchFor(doc);
		Statements sts_ann = rs.searchFor(annotation);
		
		AdvancedLinkedHashMap alhm = new AdvancedLinkedHashMap(sts_ann);
		LinkedHashMap<String, List<WTriple>> map = alhm.getMap();
		List<WTriple> triples = new LinkedList<WTriple>();
		map.put("mypredicate", triples);
		alhm.setMap(map);
		
		Assert.assertNull(alhm.getValue("mypredicate"));
		
		Assert.assertEquals("{mypredicate=[]}", alhm.toString());
	
		try{
			alhm.writeStatement("mypredicate_unknown", "myobject");
			Assert.fail("Can not write a predicate without an existing previous WTriple");
		}catch (WebLabCheckedException e) {
			// Normal
		}
		try{
			alhm.writeStatement(null, "myobject");
			Assert.fail("Can not write a null predicate ");
		}catch (WebLabCheckedException e) {
			// Normal
		}
		try{
			alhm.writeStatement("mypredicate_unknown", null);
			Assert.fail("Can not write a null object ");
		}catch (WebLabCheckedException e) {
			// Normal
		}
		try{
			alhm.writeStatements(new String[]{}, new String[]{}, annotation);
			Assert.fail("Can not write empty predicates");
		}catch (WebLabCheckedException e) {
			// Normal
		}
		try{
			alhm.writeStatements(new String[]{"predicates"}, new String[]{}, annotation);
			Assert.fail("Can not write empty objects");
		}catch (WebLabCheckedException e) {
			// Normal
		}
		try{
			alhm.writeStatements(new String[]{"predicates"}, new String[]{"object","object"}, annotation);
			Assert.fail("Can not write object without predicate");
		}catch (WebLabCheckedException e) {
			// Normal
		}
		try{
			alhm.writeStatements(null, null, annotation);
			Assert.fail("Can not write null predicate and object");
		}catch (WebLabCheckedException e) {
			// Normal
		}
		try{
			alhm.writeStatements(new String[]{"predicates"}, null, annotation);
			Assert.fail("Can not write object without predicate without object");
		}catch (WebLabCheckedException e) {
			// Normal
		}
		WTriple test = new WTriple(
				new Triple(Node.createURI("my://node"),Node.createURI("my://predicate_unknown"),Node.createLiteral("myObject")),
				"hop", annotation, new LinkedList<Resource>());
		test.setAnnotation(annotation);
		
		triples.add(test);
		
		Assert.assertFalse("{mypredicate=[]}".equals(alhm.toString()));
		
		alhm.getStatements().clear();
		
		alhm.writeStatements(new String[]{"my://predicate_unknown"}, new String[]{"myobject"}, annotation);
		
		Assert.assertEquals(2, alhm.getStatements().size());
		
		AdvancedLinkedHashMap.write(triples, new PieceOfKnowledge(), sts_ann, new HashMap<String, String>());
		
		Assert.assertEquals(1, sts_ann.size());
		
		PieceOfKnowledge pokd = new PieceOfKnowledge();
		pokd.setData(new Object());
		AdvancedLinkedHashMap.write(triples, pokd, sts_ann, new HashMap<String, String>());
		
		Assert.assertEquals(1, sts_ann.size());
		
		alhm = new AdvancedLinkedHashMap(sts_doc, "temp://uri");
		alhm.put(test);
		alhm.put(new WTriple("test", "test", "test", null, null, annotation));
				
		alhm.writeStatements(new String[]{"my://predicate_unknown2"}, new String[]{"uri://uri#test"}, null);
		
		Assert.assertEquals(3, alhm.allPredicates().size());
		
		
		List<WTriple> list = new LinkedList<WTriple>();
		list.add(new WTriple(null, "test", "test", null, null, annotation));
		
		try{
			AdvancedLinkedHashMap.write(list, pokd, sts_ann, new HashMap<String, String>());
			Assert.fail("Wtriple has no subject");
		}catch(WebLabCheckedException e){
			// Normal
		}
		list.clear();
		list.add(new WTriple("test", null, "test", null, null, annotation));
		try{
			AdvancedLinkedHashMap.write(list, pokd, sts_ann, new HashMap<String, String>());
			Assert.fail("Wtriple has no subject");
		}catch(WebLabCheckedException e){
			// Normal
		}
		list.clear();
		list.add(new WTriple("test", "test", null, null, null, annotation));
		try{
			AdvancedLinkedHashMap.write(list, pokd, sts_ann, new HashMap<String, String>());
			Assert.fail("Wtriple has no subject");
		}catch(WebLabCheckedException e){
			// Normal
		}
		
		alhm.getMap().clear();
		
		Assert.assertTrue(alhm.getStatements().isEmpty());
		alhm.writeStatements(new String[]{"my://predicate_unknown2"}, new String[]{"uri://uri#test"}, new PieceOfKnowledge());
		
		Assert.assertEquals(1, alhm.allPredicates().size());
	}
}
