/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 *  
 * Copyright (C) 2004 - 2012 Cassidian, an EADS company
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */
package org.ow2.weblab.core.helper.impl.parser;

import java.io.File;
import java.lang.reflect.Constructor;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;

import junit.framework.Assert;

import org.junit.BeforeClass;
import org.junit.Test;
import org.ow2.weblab.core.extended.exception.WebLabCheckedException;
import org.ow2.weblab.core.extended.jaxb.WebLabMarshaller;
import org.ow2.weblab.core.extended.ontologies.DublinCore;
import org.ow2.weblab.core.extended.ontologies.RDFS;
import org.ow2.weblab.core.extended.ontologies.WebLabProcessing;
import org.ow2.weblab.core.helper.impl.AdvancedSelector;
import org.ow2.weblab.core.helper.impl.RDFSelectorFactory;
import org.ow2.weblab.core.helper.impl.Statements;
import org.ow2.weblab.core.helper.test.ResourcesCreator;
import org.ow2.weblab.core.model.Annotation;

public class TypedValuesTest {


	@BeforeClass
	public static void buildResources() throws WebLabCheckedException {
		if (!ResourcesCreator.ANNOTATION.exists()) {
			ResourcesCreator.buildTestResources();
		}
	}


	@Test
	public void resetParsers() throws SecurityException, IllegalArgumentException {
		final Object result = ReflectChanger.invoke(null, Parsers.class, "loadParsers", true, new Object[0]);
		ReflectChanger.setField(null, Parsers.class, "parsers", result, true);
	}


	@Test
	public void testTyped() throws WebLabCheckedException, SecurityException, IllegalArgumentException {
		this.resetParsers();
		Parsers.strict = false;
		Annotation a = new WebLabMarshaller().unmarshal(new File("./target/annotation.xml"), Annotation.class);
		final AdvancedSelector rs = RDFSelectorFactory.getSelector(true);
		final Statements wm = rs.searchFor(a);
		final LinkedList<Date> ld = wm.getTypedValues(null, WebLabProcessing.HAS_GATHERING_DATE, Date.class);
		Assert.assertEquals(1, ld.size());
		final String s = wm.getTypedValue(null, RDFS.LABEL, String.class);
		Assert.assertEquals("empty", s);

	}


	@Test
	public void testRDFSplit() {
		// in first
		final RDFDateTypeParser rdatep = new RDFDateTypeParser();
		final Date d = rdatep.parseValue("12 888888");
		Assert.assertNull(d);
		String[] values = rdatep.split("\"test");
		Assert.assertEquals("\"test", values[0]);
		Assert.assertEquals("", values[1]);
		Assert.assertEquals("", values[2]);
		values = rdatep.split("\"test\"@fr");
		Assert.assertEquals("test", values[0]);
		Assert.assertEquals("", values[1]);
		Assert.assertEquals("fr", values[2]);
		values = rdatep.split("\"test\"^^inte");
		Assert.assertEquals("test", values[0]);
		Assert.assertEquals("inte", values[1]);
		Assert.assertEquals("", values[2]);
		values = rdatep.split("\"test\"^^inte@fr");
		Assert.assertEquals("test", values[0]);
		Assert.assertEquals("inte", values[1]);
		Assert.assertEquals("fr", values[2]);
	}


	@Test
	public void testRDFDateTypeParser() {
		Parsers.strict = false;
		final RDFDateTypeParser rdatep = new RDFDateTypeParser();
		String date = "\"\n" + "			02/13/2002\n" + "		\"^^xsd:date";
		Date t = rdatep.parseValue(date);
		final SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		Assert.assertNotNull(t);
		Assert.assertEquals("2002-02-13", sdf.format(t));
		final RDFIntegerTypeParser rintp = new RDFIntegerTypeParser();
		final Integer i = rintp.parseValue(date);
		Assert.assertNull(i);
		date = "\"\n" + "			2009-09-09T17:29:01+0200\n" + "		\"^^xsd:date";
		t = rdatep.parseValue(date);
		Assert.assertNotNull(t);
		date = "\"\n" + "			200x9-09-09Tx17:29:01+02x00\n" + "		\"^^xsd:date";
		t = rdatep.parseValue(date);
		Assert.assertNull(t);
	}


	@Test
	public void testRDFIntegerTypeParser() {
		Parsers.strict = false;
		final RDFIntegerTypeParser rintp = new RDFIntegerTypeParser();
		String integer = "\"\n" + "			213545\n" + "		\"^^xsd:int@fr";

		final Integer i = rintp.parseValue(integer);
		if (i == null) {
			Assert.fail("RDF Integer Type parser returns null instead of 213545.");
		}
		Assert.assertNotNull(i);
		Assert.assertEquals(213545, i.intValue());
		final RDFDateTypeParser rdatep = new RDFDateTypeParser();
		final Date t = rdatep.parseValue(integer);
		Assert.assertNull(t);
		integer = "213545";
		final Integer z = rintp.parseValue(integer);
		Assert.assertNotNull(z);
		Assert.assertEquals(213545, z.intValue());

	}


	@Test
	public void testRDFFloatTypeParser() {
		Parsers.strict = true;
		final RDFFloatTypeParser rintp = new RDFFloatTypeParser();
		String f = "\"\n" + "			213.545\n" + "		\"^^xsd:float@fr";

		Float i = rintp.parseValue(f);
		if (i == null) {
			Assert.fail("RDF Float Type parser returns null instead of 213.545.");
		}
		Assert.assertNotNull(i);
		Assert.assertEquals(213.545f, i.floatValue(), 0.000001f);
		Parsers.strict = false;
		f = "-0.2215454";
		i = rintp.parseValue(f);
		Assert.assertNotNull(i);
		f = "-0.sqfydgqs";
		i = rintp.parseValue(f);
		Assert.assertNull(i);
	}


	private Statements loadTripleMap() throws WebLabCheckedException {
		Annotation a = new WebLabMarshaller().unmarshal(ResourcesCreator.ANNOTATION, Annotation.class);
		final AdvancedSelector rs = RDFSelectorFactory.getSelector(true);
		return rs.searchFor(a);
	}


	@Test
	public void testNotStrictParsers() throws WebLabCheckedException {
		Parsers.strict = false;

		final Statements wm = this.loadTripleMap();
		Assert.assertEquals(1, wm.size());
		// Non strict mode on a list of result
		final String i = wm.getTypedValue(null, RDFS.LABEL, String.class);
		Assert.assertNotNull(i);
		final LinkedList<String> li = wm.getTypedValues(null, RDFS.LABEL, String.class);
		Assert.assertEquals(1, li.size());
		final Integer bi = wm.getTypedValue(null, WebLabProcessing.HAS_GATHERING_DATE, Integer.class);
		Assert.assertNull(bi);
	}


	@Test
	public void testStrictParsers() throws WebLabCheckedException {
		Parsers.strict = true;
		final Statements wm = this.loadTripleMap();
		Assert.assertEquals(1, wm.size());
		// Non strict mode on a list of result
		final Integer i = wm.getTypedValue("http://example.org/buecher/baum", DublinCore.TITLE_PROPERTY_NAME, Integer.class);
		Assert.assertNull(i);
		final LinkedList<Integer> li = wm.getTypedValues("http://example.org/buecher/baum", DublinCore.TITLE_PROPERTY_NAME, Integer.class);
		Assert.assertEquals(0, li.size());
		final Integer bi = wm.getTypedValue("http://example.org/buecher/baum", DublinCore.DESCRIPTION_PROPERTY_NAME, Integer.class);
		Assert.assertNull(bi);
	}


	@Test
	public void testParserFail() throws WebLabCheckedException {
		final Statements wm = this.loadTripleMap();
		final Double d = wm.getTypedValue("http://example.org/buecher/baum", DublinCore.TITLE_PROPERTY_NAME, Double.class);
		Assert.assertNull(d);
		final Boolean b = wm.getTypedValue("http://example.org/buecher/baum", DublinCore.TITLE_PROPERTY_NAME, Boolean.class);
		Assert.assertNull(b);
		final Date n = wm.getTypedValue("http://example.org/buecher/baum", DublinCore.TITLE_PROPERTY_NAME, Date.class);
		Assert.assertNull(n);
	}


	@Test
	public void testNullParameters() throws WebLabCheckedException {
		final Statements wm = this.loadTripleMap();
		final Double d = wm.getTypedValue(null, DublinCore.TITLE_PROPERTY_NAME, Double.class);
		Assert.assertNull(d);
		final Boolean b = wm.getTypedValue("http://example.org/buecher/baum", null, Boolean.class);
		Assert.assertNull(b);
		final Character c = wm.getTypedValue("http://example.org/buecher/baum", null, null);
		Assert.assertNull(c);
		final List<Double> ld = wm.getTypedValues(null, DublinCore.TITLE_PROPERTY_NAME, Double.class);
		Assert.assertEquals(0, ld.size());
		final List<Boolean> lb = wm.getTypedValues("http://example.org/buecher/baum", null, Boolean.class);
		Assert.assertEquals(0, lb.size());
		final List<Character> lc = wm.getTypedValues("http://example.org/buecher/baum", null, null);
		Assert.assertEquals(0, lc.size());
	}


	@Test
	public void testInvokeParser() throws ClassCastException, ParserConfigurationException {
		try {
			final Constructor<?> c = Parsers.class.getDeclaredConstructors()[0];
			c.setAccessible(true);
			c.newInstance(new Object[] {});
			Assert.fail();
		} catch (final Exception e) {
			// An exception should have been thrown
		}
		TypeParser<?> tp = Parsers.getParser(String.class);
		Assert.assertNotNull(tp);
	}

}
