/**
* WEBLAB: Service oriented integration platform for media mining and intelligence applications
*  
* Copyright (C) 2004 - 2011 Cassidian, an EADS company
* 
* This library is free software; you can redistribute it and/or modify it under the terms of
* the GNU Lesser General Public License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
* See the GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License along with this
* library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
* Floor, Boston, MA 02110-1301 USA
*/

package org.ow2.weblab.core.helper.impl;

import org.apache.commons.logging.LogFactory;
import org.ow2.weblab.core.extended.ontologies.RDF;
import org.ow2.weblab.core.extended.ontologies.WebLab;
import org.ow2.weblab.core.extended.ontologies.WebLabModel;
import org.ow2.weblab.core.extended.util.PoKUtil;
import org.ow2.weblab.core.model.Annotation;
import org.ow2.weblab.core.model.Audio;
import org.ow2.weblab.core.model.ComposedQuery;
import org.ow2.weblab.core.model.ComposedResource;
import org.ow2.weblab.core.model.Document;
import org.ow2.weblab.core.model.Image;
import org.ow2.weblab.core.model.LinearSegment;
import org.ow2.weblab.core.model.LowLevelDescriptor;
import org.ow2.weblab.core.model.MediaUnit;
import org.ow2.weblab.core.model.PieceOfKnowledge;
import org.ow2.weblab.core.model.Query;
import org.ow2.weblab.core.model.Resource;
import org.ow2.weblab.core.model.ResultSet;
import org.ow2.weblab.core.model.Segment;
import org.ow2.weblab.core.model.SimilarityQuery;
import org.ow2.weblab.core.model.SpatialSegment;
import org.ow2.weblab.core.model.StringQuery;
import org.ow2.weblab.core.model.TemporalSegment;
import org.ow2.weblab.core.model.Text;
import org.ow2.weblab.core.model.Video;

import com.hp.hpl.jena.rdf.model.impl.PropertyImpl;
import com.hp.hpl.jena.rdf.model.impl.ResourceImpl;


/**
 * This is an implementation of the <code>JenaResourceHelper</code>. 
 * It loads every <code>Annotations</code> that are on this object and in the inner
 * <code>Resources</code>. It adds the structure statements in the generated inner model. 
 * If the loaded <code>Resource</code> is an <code>Annotation</code>, its
 * content won't be loaded.
 * 
 * @author EADS WebLab Team
 * @date 2009-02-10
 */
public class JenaResourceStructureHelperExtended extends JenaResourceHelper {

	/**
	 * Constructor 
	 * @param res 
	 */
	public JenaResourceStructureHelperExtended(Resource res) {
		super(res);
	}

	/**
	 * Method recursively called to add every resources in the model and in the map.
	 * 
	 * @param res
	 *            The resource to add.
	 */
	@Override
	protected void addResource(final Resource res) {
		this.resourcesMap.put(res.getUri(), res);
		for (final Annotation annot : res.getAnnotation()) {
			this.bejh.createResStat(res.getUri(), WebLabModel.HAS_ANNOTATION, annot.getUri());
			this.bejh.createResStat(annot.getUri(), WebLab.IN_RESOURCE_ANNOTATION, res.getUri());
			this.addResource(annot);
		}
		for (final LowLevelDescriptor lld : res.getDescriptor()) {
			this.bejh.createResStat(res.getUri(), WebLabModel.HAS_LOW_LEVEL_DESCRIPTOR, lld.getUri());
			this.bejh.createResStat(lld.getUri(), WebLab.IN_RESOURCE_LLD, res.getUri());
			this.addResource(lld);
		}

		if (res instanceof MediaUnit) {
			this.addMediaUnit((MediaUnit) res);
		} else if (res instanceof PieceOfKnowledge) {
			this.addPieceOfKnowledge((PieceOfKnowledge) res);
		} else if (res instanceof Query) {
			this.addQuery((Query) res);
		} else if (res instanceof ResultSet) {
			this.addResultSet((ResultSet) res);
		} else if (res instanceof ComposedResource) {
			this.addComposedResource((ComposedResource) res);
		} else if (res instanceof LowLevelDescriptor) {
			this.bejh.createResStat(res.getUri(), RDF.TYPE, WebLabModel.getUriFromResource(LowLevelDescriptor.class));
		} else {
			if (!res.getClass().equals(Resource.class)) {
				LogFactory.getLog(this.getClass()).warn("Resource type unknown: " + res.getClass().getCanonicalName());
			}
			this.bejh.createResStat(res.getUri(), RDF.TYPE, WebLabModel.getUriFromResource(Resource.class));
		}

	}

	/**
	 * @param query
	 *            The query to add.
	 */
	protected void addQuery(final Query query) {
		if (query instanceof StringQuery) {
			this.bejh.createResStat(query.getUri(), RDF.TYPE, WebLabModel.getUriFromResource(StringQuery.class));
		} else if (query instanceof SimilarityQuery) {
			this.addSimilarityQuery((SimilarityQuery) query);
		} else if (query instanceof ComposedQuery) {
			this.addComposedQuery((ComposedQuery) query);
		} else {
			LogFactory.getLog(this.getClass()).warn("Unknown type of Query: " + query.getClass().getCanonicalName());
			this.bejh.createResStat(query.getUri(), RDF.TYPE, WebLabModel.getUriFromResource(Query.class));
		}
	}

	protected void addSimilarityQuery(final SimilarityQuery squery) {
		this.bejh.createResStat(squery.getUri(), RDF.TYPE, WebLabModel.getUriFromResource(SimilarityQuery.class));
		for (final Resource resource : squery.getResource()) {
			this.bejh.createResStat(squery.getUri(), WebLab.HAS_RESOURCE_SIM_QUERY, resource.getUri());
			this.bejh.createResStat(resource.getUri(), WebLab.IN_SIMILARITY_QUERY, squery.getUri());
			this.addResource(resource);
		}
	}

	protected void addComposedQuery(final ComposedQuery cquery) {
		this.bejh.createResStat(cquery.getUri(), RDF.TYPE, WebLabModel.getUriFromResource(ComposedQuery.class));
		for (final Query query : cquery.getQuery()) {
			this.bejh.createResStat(cquery.getUri(), WebLab.HAS_QUERY_IN_COMPOSED, query.getUri());
			this.bejh.createResStat(query.getUri(), WebLabModel.IN_COMPOSED_QUERY, cquery.getUri());
			this.addResource(query);
		}
	}

	protected void addResultSet(final ResultSet resultset) {
		this.bejh.createResStat(resultset.getUri(), RDF.TYPE, WebLabModel.getUriFromResource(ResultSet.class));
		for (final Resource resource : resultset.getResource()) {
			this.bejh.createResStat(resultset.getUri(), WebLab.HAS_RESOURCE_RESULT, resource.getUri());
			this.bejh.createResStat(resource.getUri(), WebLab.IN_RESULT_SET_RESOURCE, resultset.getUri());
			this.addResource(resource);
		}
		final PieceOfKnowledge pok = resultset.getPok();
		if (pok != null) {
			this.bejh.createResStat(resultset.getUri(), WebLab.HAS_POK_RESULT, pok.getUri());
			this.bejh.createResStat(pok.getUri(), WebLab.IN_RESULT_SET_POK, resultset.getUri());
			this.addResource(pok);
		}
	}

	protected void addComposedResource(final ComposedResource composedResource) {
		for (final Resource resource : composedResource.getResource()) {
			this.bejh.createResStat(composedResource.getUri(), WebLab.HAS_RESOURCE, resource.getUri());
			this.bejh.createResStat(resource.getUri(), WebLab.IN_COMPOSED_RESOURCE, composedResource.getUri());
			this.addResource(resource);
		}
		this.bejh.createResStat(composedResource.getUri(), RDF.TYPE, WebLabModel.getUriFromResource(ComposedResource.class));
	}


	@Override
	protected void addPieceOfKnowledge(final PieceOfKnowledge pok) {
		final String rdf = PoKUtil.getPoKData(pok);
		if (!rdf.trim().equals("")) {
			this.loadFromRdfXml(rdf, true);
		}
		this.bejh.createResStat(pok.getUri(), RDF.TYPE, WebLabModel.getUriFromResource(Annotation.class));
	}

	/**
	 * @param seg
	 *            The segment to add.
	 */
	protected void addSegment(Segment seg) {
		if (seg instanceof LinearSegment) {
			this.bejh.createResStat(seg.getUri(), RDF.TYPE, WebLabModel.LINEAR_SEGMENT);
			this.bejh.getModel().addLiteral(new ResourceImpl(seg.getUri()), new PropertyImpl(WebLabModel.START_AT), ((LinearSegment)seg).getStart());
			this.bejh.getModel().addLiteral(new ResourceImpl(seg.getUri()), new PropertyImpl(WebLabModel.END_AT), ((LinearSegment)seg).getEnd());
			
			
		} else if (seg instanceof TemporalSegment) {
			this.bejh.createResStat(seg.getUri(), RDF.TYPE, "http://weblab-project.org/core/model/class/TemporalSegment");
		} else if (seg instanceof SpatialSegment) {
			this.bejh.createResStat(seg.getUri(), RDF.TYPE, "http://weblab-project.org/core/model/class/SpatialSegment");
		} else {
			LogFactory.getLog(this.getClass()).warn("Unknown type of Segment: " + seg.getClass().getCanonicalName());
			this.bejh.createResStat(seg.getUri(), RDF.TYPE, "http://weblab-project.org/core/model/class/Segment");
		}
	}

	/**
	 * @param mediaunit
	 *            The mediaUnit to add.
	 */
	protected void addMediaUnit(final MediaUnit mediaunit) {
		for (final Segment seg : mediaunit.getSegment()) {
			this.bejh.createResStat(mediaunit.getUri(), WebLabModel.HAS_SEGMENT, seg.getUri());
			this.bejh.createResStat(seg.getUri(), WebLab.IN_MEDIA_UNIT, mediaunit.getUri());
			this.addSegment(seg);
		}

		if (mediaunit instanceof Document) {
			this.addDocument((Document) mediaunit);
		} else if (mediaunit instanceof Text) {
			this.bejh.createResStat(mediaunit.getUri(), RDF.TYPE, WebLabModel.getUriFromResource(Text.class));
		} else if (mediaunit instanceof Image) {
			this.bejh.createResStat(mediaunit.getUri(), RDF.TYPE, WebLabModel.getUriFromResource(Image.class));
		} else if (mediaunit instanceof Audio) {
			this.bejh.createResStat(mediaunit.getUri(), RDF.TYPE, WebLabModel.getUriFromResource(Audio.class));
		} else if (mediaunit instanceof Video) {
			this.bejh.createResStat(mediaunit.getUri(), RDF.TYPE, WebLabModel.getUriFromResource(Video.class));
		} else {
			LogFactory.getLog(this.getClass()).warn("Unknown type of MediaUnit: " + mediaunit.getClass().getCanonicalName());
			this.bejh.createResStat(mediaunit.getUri(), RDF.TYPE, WebLabModel.getUriFromResource(MediaUnit.class));
		}
	}

	/**
	 * @param document
	 */
	protected void addDocument(final Document document) {
		for (final MediaUnit mediaunit : document.getMediaUnit()) {
			this.bejh.createResStat(document.getUri(), WebLab.HAS_MEDIA_UNIT, mediaunit.getUri());
			this.bejh.createResStat(mediaunit.getUri(), WebLab.IN_COMPOSED_UNIT, document.getUri());
			this.addResource(mediaunit);
		}
		
		this.bejh.createResStat(document.getUri(), RDF.TYPE, WebLabModel.getUriFromResource(Document.class));
		 
	}

}
